/****************************************************************/
/*  FT-817 External Display                                     */
/*                                                              */
/*  Author:    Hiroshi Shirahara(7J3AOZ)                        */
/*  Date:      2014/10/30                                       */
/*                                                              */
/*  This code is in the public domain.                          */
/*                                                              */
/*  This sketch requires the following libraries.               */
/*                                                              */
/*  Adafruit-RGB-LCD-Shield-Library                             */
/*  https://github.com/adafruit/Adafruit-RGB-LCD-Shield-Library */ 
/*                                                              */
/*  arduino_ft817                                               */
/*  https://github.com/8cH9azbsFifZ/arduino_ft817               */
/*                                                              */
/*  Metro-Arduino-Wiring                                        */
/*  https://github.com/thomasfredericks/Metro-Arduino-Wiring    */
/*                                                              */
/****************************************************************/

#include <Wire.h>                         // I2Cライブラリのヘッダーファイル
#include <Metro.h>                        // 「Metro」ライブラリのヘッダーファイル

#include <Adafruit_MCP23017.h>            // LCDシールドライブラリのヘッダーファイル
#include <Adafruit_RGBLCDShield.h>

#include <FT817.h>                        // FT-817ライブラリのヘッダーファイル

// ------- RGB LCDシールド関連定数定義

#define RED 0x1                           // LCDのバックライトが赤色
#define GREEN 0x2                         // LCDのバックライトが緑色
#define YELLOW 0x3                        // LCDのバックライトが黄色
#define BLUE 0x4                          // LCDのバックライトが青色
#define VIOLET 0x5                        // LCDのバックライトが紫色
#define TEAL 0x6                          // LCDのバックライトが水色
#define WHITE 0x7                         // LCDのバックライトが白色

// ------- FT-817ライブラリ関連定数

#define FT817_SPEED 38400                 // FT-817との通信ボーレート

// ------- ライブラリクラスインスタンス生成

// Metroライブラリのインスタンス生成
Metro timer1 = Metro(200);                // タイマー間隔を200msとして生成

// LCDシールドライブラリのインスタンス生成
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// FT-817ライブラリのインスタンス生成
FT817 ft817(&Serial);                     // ハードウェアシリアルを指定して生成

// ------- グローバル変数定義

char rig_mode[5];                          // FT-817のモードを格納
long rig_freq;                             // FT-817の周波数を格納
int rig_width;                             // FT-817のフィルタのバンド幅を格納(0:WIDE,1:NARROW)

unsigned int rig_s;                        // Sメーターの値を格納
char rig_s_str[4];                         // Sメーターの値を文字列で格納

boolean rig_tx;                            // 送受信状態を格納                            

unsigned int rig_ant;                      // 選択されているアンテナコネクタの種別を格納
unsigned int rig_pwr;                      // 選択されている送信出力を格納
unsigned int rig_swr;                      // SWRの値を格納

int state;                                 // 現在選択している表示モード（0～4）
                                           // モード 0 : Sメータ表示
                                           // モード 1 : SWR表示
                                           // モード 2 : フィルタ幅表示
                                           // モード 3 : 選択されたアンテナ表示
                                           // モード 4 : 送信出力表示

int lcd_color;                             // 現在選択しているLCDのバックライト色（1～7）
                                           // 数値の意味は「RGB LCDシールド関連定数定義」
                                           // を参照

// ------ 初期化

void setup() {

  state = 0;                               // LCDの表示モードを初期化

  lcd_color = WHITE;                       // LCDのバックライト色の初期値を白色(7)にする
  
  lcd.begin(16, 2);                        // LCDシールド初期化
  lcd.setBacklight(lcd_color);             // LCDのバックライト色をセット

  lcd.setCursor(0,0);                      // LCDのカーソル位置を(0,0)にセット
  lcd.print("Connecting....");             // 起動メッセージを表示

  ft817.begin(FT817_SPEED);                // FT-817ライブラリ初期化
  delay(1000);                             // FT-817ライブラリ初期化待ち（1秒） 
  read_rig();                              // FT-817からのデータ読み込み  

  tone(8,400,100);                         // 起動を示すためにブザーを3回鳴らす
  lcd.clear();                             // LCD表示クリア

}

// ------ メインループ

void loop ()
{    

  uint8_t buttons;                          // LCDシールドのボタンデータを格納
  
  if (timer1.check() == 1) {                // タイマーが時間になったか？
    timer1.reset();                         // タイマーをリセット
    read_rig();                             // FT-817からのデータ読み込み
    disp_rig();                             // FT-817のデータ表示
  }

  buttons = lcd.readButtons();              // LCDシールドのボタンデータを取得

  if (buttons) {                            // どれかのボタンが押されているか？
    if (buttons & BUTTON_UP) {              // 上ボタンが押された場合
      tone(8,400,100);                      // ボタンの押下を示すためにブザーを鳴らす
      state = state - 1;                    // 表示モードを示す数値を減算する
      if (state < 0) state = 4;             // 表示モードが0以下になったら表示モードを4にする
      }
    else if (buttons & BUTTON_DOWN) {       // 下ボタンが押された場合
      tone(8,400,100);                      // ボタンの押下を示すためにブザーを鳴らす
      state = state + 1;                    // 表示モードを示す数値を加算する
      if (state > 4) state = 0;             // 表示モードが4以上になったら表示モードを0にする
      }
    else if (buttons & BUTTON_LEFT) {       // 左ボタンが押された場合
      tone(8,400,100);                      // ボタンの押下を示すためにブザーを鳴らす
      lcd_color = lcd_color - 1;            // バックライト色を示す値を1減算する
      if (lcd_color < 1) lcd_color = 7;     // バックライト色が1以下なら7にする
      lcd.setBacklight(lcd_color);          // LCDのバックライト色をセット
      }
    else if (buttons & BUTTON_RIGHT) {      // 右ボタンが押された場合
      tone(8,400,100);                      // ボタンの押下を示すためにブザーを鳴らす
      lcd_color = lcd_color + 1;            // バックライト色を示す値を1加算する
      if (lcd_color > 7) lcd_color = 1;     // バックライト色が7以上なら1にする
      lcd.setBacklight(lcd_color);          // LCDのバックライト色をセット
      }
    if (buttons & BUTTON_SELECT) {          // SELECTボタンが押された場合
      ft817.switchVFO();                    // FT-817のVFO A/Bを切り替える
      }
    else {
      tone(8,200,100);                      // 間違ったボタンの押下を示すためにブザーを鳴らす
      }
    delay(50);                              // ボタンのチャタリング防止のため50msウェイト
    }
  
}

// ------ FT-817からのデータ読み込み

void read_rig()
{
  
  while(1) {
    
    rig_freq = ft817.getFreqMode(rig_mode);  // FT-817から周波数とモードを読み込み
    if (rig_freq != 0) break;                // 正常に読み込めたらループを抜ける

    }

  if (state == 0) {                          // 表示モードが0の場合
    rig_tx = ft817.txState2();
    rig_s = ft817.getRxStatus(rig_s_str);    // FT-817のSメーターの値を取得 
    rig_s = rig_s & 0x0f;
    }
  else if (state == 1) {                     // 表示モードが1の場合
    rig_tx = ft817.txState2();
    rig_swr = ft817.getSWR();                // SWRを取得
    }
  else if (state == 3) {                     // 表示モードが3の場合
    rig_ant = ft817.getAnt() & 0xf;          // 選択されているアンテナを取得
    }
  else if (state == 4) {                     // 表示モードが4の場合
    rig_pwr = ft817.getPWR();                // 選択されている送信出力を取得
    }

} 

// ------ FT-817のデータ表示

void disp_rig()
{
   
    display_freq_mode();                    // FT-817の周波数・モード表示

    if (state == 0) {                       // 表示モードが0の場合
      display_s();                          // Sメータ表示
      }
    else if (state == 1) {                  // 表示モードが1の場合
      display_swr();                        // SWR表示
      }
    else if (state == 2) {                  // 表示モードが2の場合
      display_width();                      // フィルタ幅表示
      }
    else if (state == 3) {                  // 表示モードが3の場合
      display_ant();                        // 選択されたアンテナ表示
      }
    else if (state == 4) {                  // 表示モードが4の場合
      display_pwr();                        // 送信出力表示
      }
      
}

// ------ FT-817の周波数・モード表示

void display_freq_mode()
{

  long freq;                                 // 周波数データ計算用変数 
  int mhz, khz, hz;
  
  char mode_buf[5];                          // モード文字列編集用バッファ
  char disp_buf[17];                         // LCD表示用バッファ
  
  rig_width = 0;                             // バンド幅の初期値に0をセット
  
  // ------- 得られた周波数データからMHz.kHz,Hzを求める
  
  freq = rig_freq * 10;                      // 得られた周波数をHz単位にする

  mhz = freq / 1000000;                      // MHzを得る
  khz = (freq % 1000000)/1000;               // KHzを得る 
  hz = freq % 1000;                          // Hzを得る

  // ------- 得られたモードを表示用に編集する

  if (strcmp("LSB",rig_mode) == 0) {        // LSB(フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("USB",rig_mode) == 0) {   // USB（フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("CW",rig_mode) == 0) {    // CW（フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("CWR",rig_mode) == 0) {   // CWリバース（フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("AM",rig_mode) == 0) {    // AM（フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("FM",rig_mode) == 0) {    // FM（フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("FMN",rig_mode) == 0) {   // FM（フィルターナロー）
    strcpy(mode_buf,rig_mode);
    rig_width = 1;
    }
  else if (strcmp("DIG",rig_mode) == 0) {   // デジタル（フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("PKT",rig_mode) == 0) {   // パケット（フィルターワイド）
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("CWN",rig_mode) == 0) {   // CW（フィルターナロー）
    strcpy(mode_buf,"CW");
    rig_width = 1;
    }
  else if (strcmp("CWRN",rig_mode) == 0) {   // CWリバース（フィルターナロー）
    strcpy(mode_buf,"CWR");
    rig_width = 1;
    }
  else if (strcmp("DIGN",rig_mode) == 0) {   // デジタル（フィルターナロー）
    strcpy(mode_buf,"DIG");
    rig_width = 1;
    }
  else if (rig_mode[0] == 0) {               // ワイドFM(FM放送など）
    strcpy(mode_buf,"WFM");
    }

  // 表示用バッファに周波数とモードをセットしLCDの1行目に表示
  
  sprintf(disp_buf,"%03d.%03d.%03d  %3s", mhz, khz, hz, mode_buf);
  lcd.setCursor(0, 0);
  lcd.print(disp_buf);

}

// ------ FT-817のSメータ表示

void display_s()
{
  
  int i;                                     // ループカウンタ
  
  char s_buf[10];                            // Sメーターバー表示文字列編集用バッファ
  char disp_buf[17];                         // LCD表示用バッファ

  s_buf[0] = 0x0;                            // バッファ初期化

  if (rig_s != 0) {                          // バー表示用文字列生成
    for (i = 0; i <= rig_s ; i = i + 1) {
      if (i == 9) break;
      s_buf[i] = 0xff;
      s_buf[i+1] = 0x0;
      }
    }

  // 表示用バッファに周波数とモードをセットしLCDの2行目に表示
 
  if (rig_tx == false) {
    if (rig_s <= 9) {                          // Sが9以下の場合
      sprintf(disp_buf,"%-9s %3s   ", s_buf, rig_s_str);    // Sメータの値のみ表示
      }
     else {                                   // Sが9を超えている場合 
      sprintf(disp_buf,"%-9s S9 %3s", s_buf, rig_s_str);    // +**dbを表示
      }
    }
  else strcpy(disp_buf, "Transmit        ");  // 送信中の表示

  lcd.setCursor(0, 1);
  lcd.print(disp_buf);
  
}

// ------ FT-817のSWR表示
void display_swr()
{
  
  int i;                                     // ループカウンタ
  
  char swr_buf[10];                          // Sメーターバー表示文字列編集用バッファ
  char disp_buf[17];                         // LCD表示用バッファ

  swr_buf[0] = 0x0;                          // バッファ初期化

  if (rig_swr != 0) {                        // バー表示用文字列生成
    for (i = 0; i <= rig_swr ; i = i + 1) {
      if (i == 9) break;
      swr_buf[i] = 0xff;
      swr_buf[i+1] = 0x0;
      }
    }

  // 表示用バッファにSWRの値をセットしLCDの2行目に表示
 
  if (rig_tx == true) {    // 送信中の場合
      if (rig_swr == 9) {  // SWRが無限大（値が9）の場合
        sprintf(disp_buf,"SWR VERY HIGH  %1d", rig_swr);    // 「SWR VERY HIGH」表示
        }
      else {              // SWRの値を表示（0～8）
                          // 各値のおおよそのSWR値は、下記のページが参考になります
                          // http://www.bwt.jp/wiki/index.php?FT817
        sprintf(disp_buf,"SWR %-9s %1d", swr_buf, rig_swr);
        }
    }
  else strcpy(disp_buf, "Recive          ");                // 受信中の表示

  lcd.setCursor(0, 1);
  lcd.print(disp_buf);
  
}

// ------ FT-817のフィルタ幅表示
void display_width()
{

 lcd.setCursor(0, 1);                      // LCDのカーソル位置を2行も1文字目に

 if (rig_width == 0) {                     // フィルタ幅がワイドの場合
    lcd.print("FILTER : WIDE   ");
    }
 else {                                    // フィルタ幅がナローの場合
    lcd.print("FILTER : NARROW ");
    }

}

// ------ FT-817のアンテナ表示

void display_ant()
{

  char ant_buf[6];                           // アンテナ表示編集用バッファ
  char disp_buf[17];                         // LCD表示用バッファ
  
  ant_buf[0] = 0x0;                          // バッファ初期化
  
  if (rig_ant == 0) {                        // 選択されているアンテナコネクタがパネル面（BNC）の場合
    strcpy(ant_buf, "FRONT");
   }
 else {                                      // 選択されているアンテナコネクタが背面（M）の場合                 
    strcpy(ant_buf, "REAR ");
   }

  sprintf(disp_buf,"ANTENNA : %5s ", ant_buf);

  lcd.setCursor(0, 1);
  lcd.print(disp_buf);

}

// ------ FT-817の出力設定表示

void display_pwr()
{

  char pwr_buf[6];                           // 出力設定表示編集用バッファ
  char disp_buf[17];                         // LCD表示用バッファ
  
  pwr_buf[0] = 0x0;                          // バッファ初期化

  if (rig_pwr == 1) {                        // 出力がL1の場合
    strcpy(pwr_buf, "0.");
   }
  else if (rig_pwr == 2) {                   // 出力がL2の場合
    strcpy(pwr_buf, "1.0");
   }
  else if (rig_pwr == 3) {                   // 出力がL3の場合
    strcpy(pwr_buf, "2.5");
   }
  else if (rig_pwr == 4) {                   // 出力がHIの場合
    strcpy(pwr_buf, "5.0");
   }

  sprintf(disp_buf,"POWER : %3s Watt", pwr_buf);

  lcd.setCursor(0, 1);
  lcd.print(disp_buf);

}


